Package: frma
Version: 1.63.0
Date: 2017-11-08
Title: Frozen RMA and Barcode
Description: Preprocessing and analysis for single microarrays and microarray batches.
Author: Matthew N. McCall <mccallm@gmail.com>, Rafael A. Irizarry <rafa@jhu.edu>, with contributions from Terry Therneau
Maintainer: Matthew N. McCall <mccallm@gmail.com>
Depends: R (>= 2.10.0), Biobase (>= 2.6.0)
Imports: Biobase, MASS, DBI, affy, methods, oligo, oligoClasses,
        preprocessCore, utils, BiocGenerics
Suggests: hgu133afrmavecs, frmaExampleData
biocViews: Software, Microarray, Preprocessing
License: GPL (>= 2)
URL: http://bioconductor.org
git_url: https://git.bioconductor.org/packages/frma
git_branch: devel
git_last_commit: 06e0f3f
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.23
Date/Publication: 2025-11-02
NeedsCompilation: no
Packaged: 2025-11-02 22:40:52 UTC; biocbuild
