importFrom(Biobase, phenoData, experimentData)
importClassesFrom(Biobase, AnnotatedDataFrame, AssayData, eSet,
                  ExpressionSet, MIAME, Versioned, VersionedBiobase,
                  Versions)
importClassesFrom(methods, character, matrix)
importMethodsFrom(methods, initialize)
importMethodsFrom(Biobase, annotatedDataFrameFrom, exprs, se.exprs, "se.exprs<-", annotation, sampleNames)
importMethodsFrom(affy, cdfName, pm, "pm<-", pmindex, probeNames)
importMethodsFrom(oligo, backgroundCorrect)
importMethodsFrom(oligoClasses, db)
importMethodsFrom(DBI, dbGetQuery)

importFrom(Biobase, assayDataElement, assayDataElementReplace, assayDataNew)
importFrom(MASS, rlm)
importFrom(affy, bg.correct.rma, cleancdfname, ReadAffy, plotDensity)
importFrom(preprocessCore, normalize.quantiles.use.target, rcModelPLM,
           rcModelWPLM, subColSummarizeAvg, subColSummarizeMedian)
importFrom(utils, data, citation, toBibtex)
importFrom(methods, is, callNextMethod, new, setAs, as)
importFrom(BiocGenerics, weights, residuals)

export(frma, barcode, GNUSE)
exportClasses(frmaExpressionSet)
exportMethods(initialize, se.exprs, weights, residuals, randomeffects, 
              "se.exprs<-", "weights<-", "residuals<-", "randomeffects<-")
