setGeneric("randomeffects", function(object) standardGeneric("randomeffects"))
setGeneric("as.ExpressionSet", function(object) standardGeneric("as.ExpressionSet"))
setGeneric("weights<-", function(object, value) standardGeneric("weights<-"))
setGeneric("residuals<-", function(object, value) standardGeneric("residuals<-"))
setGeneric("randomeffects<-", function(object, value) standardGeneric("randomeffects<-"))
